import * as Yeoman from 'yeoman-generator';
/**
 * static name for generator and template directories
 */
export declare const GENERATOR_DIR_NAME: string, TEMPLATE_DIR_NAME: string;
/**
 * Generator argument interface
 */
export interface IGeneratorArgument extends Yeoman.ArgumentConfig {
    /** Description for the argument */
    description?: string;
    /** whether it is required */
    required?: boolean;
    /** whether it is optional */
    optional?: boolean;
    /** expected argument type */
    type?: typeof String | typeof Number | typeof Array | typeof Object;
    /** Default value for this argument */
    default?: string | number | any[] | object;
}
/**
 * Generator option interface
 */
export interface IGeneratorOption extends Omit<Yeoman.OptionConfig, 'type'> {
    /** Option name alias (example `h` is an alias for `help`) */
    alias?: string;
    /** Description for the option */
    description?: string;
    /** the option type */
    type?: typeof Boolean | typeof String | typeof Number;
    /** Default value */
    default?: boolean | string | number;
    /** Boolean whether to omit from help & usage */
    hide?: boolean;
}
/** Options common to all generators */
export interface IBaseGeneratorOptions {
    skipInstall?: boolean;
}
/** Arguments defined for all generators */
export interface IBaseGeneratorArgs {
}
/** Base options for a sub generator */
export declare type SubGeneratorOptions = IBaseGeneratorOptions & {
    parent?: BaseGenerator;
};
export declare type IGeneratorCtor = new (...args: any[]) => BaseGenerator;
/**
 * Abstract generator from which others are based
 * @extends Yeoman
 */
export declare abstract class BaseGenerator<T extends IBaseGeneratorOptions = any, K extends IBaseGeneratorArgs = any> extends Yeoman {
    options: T;
    arguments: K;
    /**
     * strips a pattern from the source template name to its final name. Useful
     * when templates include behavioral configuration files such as:
     * `package.json` and `.gitignore`. In such cases, use `__package.json` or
     * `__.gitignore` as the template name instead.
     */
    protected _templateNameStrip: string | RegExp;
    constructor(args?: string | string[], options?: any);
    /**
     * Specify arguments for the generator. Generally preferred in the constructor.
     * Arguments are assigned by name to the `this.options` hash.
     * See [documentation](http://yeoman.io/generator/Generator.html#argument) for
     * more information.
     * @param name Argument name
     * @param config Argument configuration
     *
     * ```ts
     * this.argument('prefix', {
     *   type: String,
     *   description: 'A prefix for the generated code',
     *   required: false,
     *   default: '',
     * });
     *
     * // read the argument
     * const prefix = this.options.prefix;
     * ```
     */
    argument(name: string, config: IGeneratorArgument): this;
    /**
     * Specify options for the generator. Generally preferred in teh constructor.
     * Options are assigned by camelCase name to the `this.options` hash.
     * See [documentation](http://yeoman.io/generator/Generator.html#option) for
     * more information.
     * @param name Option name
     * @param config Option configuration
     *
     * ```ts
     * this.option('language', {
     *   alias: 'l',
     *   type: String,
     *   description: 'specify output language',
     *   default: 'JavaScript',
     * });
     *
     * // read the option
     * const language = this.options.language;
     * ```
     */
    option(name: string, config: IGeneratorOption): this;
    /**
     * plain overload - not for reference
     * @param generator
     * @param options
     */
    composeWith(generator: any, options?: SubGeneratorOptions): any;
    /**
     * Prompt for user input based on [inquirer](https://www.npmjs.com/package/inquirer)
     * @param prompts prompt question configurations
     */
    prompt<A extends Yeoman.Answers = Yeoman.Answers>(prompts: Yeoman.Questions<A>): Promise<A>;
    /**
     * Implementation to support common variables when any template is written
     * with the `_writeTemplates` method.
     */
    protected _commonTemplateVars(): any;
    /**
     * perform filename interpolation by replacing `{key}` with respective values
     * and `/^__/` automatically be applied.
     * @param filename - relative name of file.
     * @param vars - interpolation variables
     */
    protected _destinationFileName(filename: string, vars: {
        name?: string;
    }): string;
    /**
     * filter templates scanned from the template directory
     * @param src the template source relative to sourceRoot
     * @return false will omit the path
     */
    protected _templateFilter?(src: string): boolean;
    /**
     * Write templates into destination
     * @param src - source directory to scan
     * @param vars - interpolation object properties
     * @param recursive - flag to write recursively
     */
    protected _writeTemplates(src: string, vars?: any, recursive?: boolean): void;
}
