"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseGenerator = exports.TEMPLATE_DIR_NAME = exports.GENERATOR_DIR_NAME = void 0;
const fs = require("fs");
const path = require("path");
const Yeoman = require("yeoman-generator");
const workspace_1 = require("./workspace");
/**
 * static name for generator and template directories
 */
_a = ['generators', 'templates'], exports.GENERATOR_DIR_NAME = _a[0], exports.TEMPLATE_DIR_NAME = _a[1];
/**
 * Abstract generator from which others are based
 * @extends Yeoman
 */
class BaseGenerator extends Yeoman {
    constructor(args, options) {
        super(args, options);
        /**
         * strips a pattern from the source template name to its final name. Useful
         * when templates include behavioral configuration files such as:
         * `package.json` and `.gitignore`. In such cases, use `__package.json` or
         * `__.gitignore` as the template name instead.
         */
        this._templateNameStrip = /^__/;
        // reset template source to the root templates directory
        const pathname = path.basename(this.sourceRoot().replace(new RegExp(`${exports.TEMPLATE_DIR_NAME}[\W]?$`), ''));
        const templatesRoot = workspace_1.resolveDir(this.sourceRoot(), exports.TEMPLATE_DIR_NAME);
        if (templatesRoot) {
            this.sourceRoot(path.join(templatesRoot, pathname));
        }
        // capture universal skip-install flag
        this.options.skipInstall = this.options.skipInstall || this.options['skip-install'] || false;
    }
    // ###################################
    // ###     Base Yeoman methods     ###
    // ###################################
    /**
     * Specify arguments for the generator. Generally preferred in the constructor.
     * Arguments are assigned by name to the `this.options` hash.
     * See [documentation](http://yeoman.io/generator/Generator.html#argument) for
     * more information.
     * @param name Argument name
     * @param config Argument configuration
     *
     * ```ts
     * this.argument('prefix', {
     *   type: String,
     *   description: 'A prefix for the generated code',
     *   required: false,
     *   default: '',
     * });
     *
     * // read the argument
     * const prefix = this.options.prefix;
     * ```
     */
    argument(name, config) {
        return super.argument(name, config);
    }
    /**
     * Specify options for the generator. Generally preferred in teh constructor.
     * Options are assigned by camelCase name to the `this.options` hash.
     * See [documentation](http://yeoman.io/generator/Generator.html#option) for
     * more information.
     * @param name Option name
     * @param config Option configuration
     *
     * ```ts
     * this.option('language', {
     *   alias: 'l',
     *   type: String,
     *   description: 'specify output language',
     *   default: 'JavaScript',
     * });
     *
     * // read the option
     * const language = this.options.language;
     * ```
     */
    option(name, config) {
        return super.option(name, config);
    }
    /**
     * Add a subgenerator run loop. Methods will be executed in parallel following
     * the standard yeoman run loop. Note that options.parent will be automatically
     * assigned as the generator from which the method is called. Options are also
     * inherited from the parent generator.
     * @param name subgenerator name registered in the GeneratorEnv
     * @param options options passed to the subgenerator instance
     */
    composeWith(name, options = {}) {
        options.parent = this;
        return super.composeWith(name, Object.assign(Object.assign({}, this.options), options), false);
    }
    /**
     * Prompt for user input based on [inquirer](https://www.npmjs.com/package/inquirer)
     * @param prompts prompt question configurations
     */
    prompt(prompts) {
        return super.prompt(prompts);
    }
    // ### Abstraction methods ###
    /**
     * Implementation to support common variables when any template is written
     * with the `_writeTemplates` method.
     */
    _commonTemplateVars() {
        return {};
    }
    /**
     * perform filename interpolation by replacing `{key}` with respective values
     * and `/^__/` automatically be applied.
     * @param filename - relative name of file.
     * @param vars - interpolation variables
     */
    _destinationFileName(filename, vars) {
        // replace placeholder file name (to not impact template repo)
        if (this._templateNameStrip) {
            filename = filename.replace(this._templateNameStrip, '');
        }
        // perform variable substitutions
        return Object.keys(vars)
            .reduce((str, key) => str.replace(`{${key}}`, vars[key]), filename);
    }
    /**
     * filter templates scanned from the template directory
     * @param src the template source relative to sourceRoot
     * @return false will omit the path
     */
    _templateFilter(src) {
        return true;
    }
    /**
     * Write templates into destination
     * @param src - source directory to scan
     * @param vars - interpolation object properties
     * @param recursive - flag to write recursively
     */
    _writeTemplates(src, vars = {}, recursive = true) {
        // extend vars with some other static properties
        vars = Object.assign(Object.assign({}, this._commonTemplateVars()), vars);
        const dir = this.templatePath(src);
        fs.readdirSync(dir)
            .filter(item => this._templateFilter(path.join(src, item)))
            .forEach(item => {
            const relPath = path.join(src, item); // relative item path
            const srcPath = path.join(dir, item); // absolute path of the source from templates
            const toFile = this._destinationFileName(item, vars); // name of output file
            const toDest = path.join(src, toFile); // relative path in destination
            const stat = fs.statSync(srcPath);
            if (stat.isFile()) {
                // write template with value interpolation
                const destPath = this.destinationPath(toDest);
                this.fs.copyTpl(srcPath, destPath, vars);
                // transpile ts => js OTF (if javascript)
                // this.__transpileResult(destPath);
            }
            else if (recursive && stat.isDirectory()) {
                this._writeTemplates(relPath, vars, recursive);
            }
        });
    }
}
exports.BaseGenerator = BaseGenerator;
