import * as YeomanEnv from 'yeoman-environment';
import { IBaseGeneratorOptions } from './core';
/**
 * Structure for a usage output for use in grid presentation
 */
export declare type IUsageTable = string[][];
/**
 * Object structure for a registered generator
 */
export interface IGeneratorUsage {
    name: string;
    description: string;
    options: IUsageTable;
    arguments: IUsageTable;
}
/**
 * Yeoman terminal adapter interface
 * @see http://yeoman.github.io/environment/TerminalAdapter.html
 */
export interface IYeomanAdapter extends YeomanEnv.Adapter {
}
export interface IGeneratorEnvOptions {
    namespace?: string;
    /** absolute directory to generator implementations */
    generatorRoot: string;
}
/**
 * Generator environment adapter to be used for generator loading, usage, and
 * invocation.
 */
export declare class GeneratorEnv {
    private options;
    /**
     * Create an instance relative to a fs path. Both a `generators` and a `templates`
     * directory must exist in a directory _above_ or _at_ the provided source
     * @param src path of file or directory from which the env is based
     * @param namespace generator namespace to use, if any
     */
    static relativeTo(src: string, namespace?: string): GeneratorEnv;
    /**
     * Provide own yeoman adapter for logging/prompting
     * @param adapter
     */
    static adapter(adapter: YeomanEnv.Adapter): void;
    private static _adapter;
    /** reference to the created yeoman environment */
    private _yo;
    constructor(options?: IGeneratorEnvOptions);
    /**
     * Reset the environment to clear registered generators
     */
    reset(): this;
    /**
     * list available generators by name. This method does not add the generators
     * to the environment, and is generally useful when validating user inputs that
     * lead to generator invocation.
     * @param name filter to a specific generator by name(s)
     */
    list(name?: string | string[]): string[];
    /**
     * Give the generators a namespace (prefix)
     * @param name the namespace to set
     */
    namespace(name: string): this;
    /**
     * Create a `yeoman-environment` instance and register generators.
     * Loads all generators when no list is provided.
     * @param generators restricts to specific generator(s)
     * @see http://yeoman.io/authoring/integrating-yeoman.html
     * @see http://yeoman.io/environment/Environment.html
     */
    load(generators?: string | string[]): this;
    /**
     * Convenience method for loading all generator implementations in the workspace
     * same as `this.load()` with no arguments
     */
    loadAll(): this;
    /**
     * get the generator usage text.
     * @param generator - filter to a specific generator
     * @see http://yeoman.io/generator/Generator.html#help
     */
    usage(generators?: string | string[]): IGeneratorUsage[];
    /**
     * Invoke a generator registered with the environment
     * @param generator generator name to run
     * @param options options passed to the generator
     * @param args additional arguments to run inside the generator
     * @todo generator args can include both options and arguments to be parsed by the generator.
     */
    run<T extends IBaseGeneratorOptions = any>(generator: string, options?: T, ...args: any[]): Promise<void>;
    /**
     * Get registered generator namespace.
     * @param name the generator without prefix
     */
    private _namespaced;
    /**
     * Parse usage text into usable table structure
     * @param usage the plain usage text emitted by yeoman
     */
    private _tabular;
    /**
     * gets yeoman env
     */
    private _env;
    /**
     * Ensures the environment has been loaded, otherwise throws error
     * @param method method name being used
     * @throws when no environment is loaded
     */
    private _ensureEnv;
}
