"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var GeneratorEnv_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorEnv = void 0;
const fs = require("fs");
const path = require("path");
const cli_1 = require("@jib/cli");
const YeomanEnv = require("yeoman-environment");
const os_1 = require("os");
const workspace_1 = require("./workspace");
const core_1 = require("./core");
/**
 * static name for generator namespace delimiter
 */
const NAMESPACE_DELIM = ':';
/**
 * Generator environment adapter to be used for generator loading, usage, and
 * invocation.
 */
let GeneratorEnv = GeneratorEnv_1 = class GeneratorEnv {
    constructor(options = {}) {
        this.options = options;
        const { generatorRoot } = options;
        if (!generatorRoot || !fs.existsSync(generatorRoot)) {
            throw new Error(`Invalid root directory for generators: '${generatorRoot}'`);
        }
    }
    /**
     * Create an instance relative to a fs path. Both a `generators` and a `templates`
     * directory must exist in a directory _above_ or _at_ the provided source
     * @param src path of file or directory from which the env is based
     * @param namespace generator namespace to use, if any
     */
    static relativeTo(src, namespace) {
        const generatorRoot = workspace_1.resolveDir(src, core_1.GENERATOR_DIR_NAME);
        return new this({
            namespace,
            generatorRoot,
        });
    }
    /**
     * Provide own yeoman adapter for logging/prompting
     * @param adapter
     */
    static adapter(adapter) {
        this._adapter = adapter;
    }
    /**
     * Reset the environment to clear registered generators
     */
    reset() {
        this._yo = null;
        return this;
    }
    /**
     * list available generators by name. This method does not add the generators
     * to the environment, and is generally useful when validating user inputs that
     * lead to generator invocation.
     * @param name filter to a specific generator by name(s)
     */
    list(name) {
        const { generatorRoot } = this.options;
        const filter = name ? [].concat(...[name]) : [];
        return fs.readdirSync(generatorRoot).filter(item => {
            const stat = fs.statSync(path.join(generatorRoot, item));
            return stat.isDirectory();
        }).filter(gen => filter.length ? ~filter.indexOf(gen) : true);
    }
    /**
     * Give the generators a namespace (prefix)
     * @param name the namespace to set
     */
    namespace(name) {
        this.options.namespace = name;
        return this;
    }
    /**
     * Create a `yeoman-environment` instance and register generators.
     * Loads all generators when no list is provided.
     * @param generators restricts to specific generator(s)
     * @see http://yeoman.io/authoring/integrating-yeoman.html
     * @see http://yeoman.io/environment/Environment.html
     */
    load(generators) {
        const env = this._env();
        const { generatorRoot } = this.options;
        this.list(generators).forEach(gen => {
            env.register(path.join(generatorRoot, gen), this._namespaced(gen));
        });
        return this;
    }
    /**
     * Convenience method for loading all generator implementations in the workspace
     * same as `this.load()` with no arguments
     */
    loadAll() {
        return this.load();
    }
    /**
     * get the generator usage text.
     * @param generator - filter to a specific generator
     * @see http://yeoman.io/generator/Generator.html#help
     */
    usage(generators) {
        this.load(generators);
        const env = this._yo;
        const usage = env.namespaces()
            .map((ns) => ({ name: ns, ctor: env.get(ns) }))
            .map((ref) => (Object.assign(Object.assign({}, ref), { instance: env.instantiate(ref.ctor, {
                arguments: null,
                options: { help: true },
            }) })))
            .map((ref) => ({
            name: ref.name.split(NAMESPACE_DELIM).pop(),
            description: ref.instance.description,
            options: this._tabular(ref.instance.optionsHelp()).slice(1),
            arguments: this._tabular(ref.instance.argumentsHelp()),
        }));
        return usage;
    }
    /**
     * Invoke a generator registered with the environment
     * @param generator generator name to run
     * @param options options passed to the generator
     * @param args additional arguments to run inside the generator
     * @todo generator args can include both options and arguments to be parsed by the generator.
     */
    // tslint:disable-next-line:max-line-length
    run(generator, options = {}, ...args) {
        return new Promise((resolve, reject) => {
            try {
                resolve(this._ensureEnv('run')
                    ._yo.run([this._namespaced(generator)].concat(args), options));
            }
            catch (e) {
                reject(e);
            }
        });
    }
    /**
     * Get registered generator namespace.
     * @param name the generator without prefix
     */
    _namespaced(name) {
        const { namespace } = this.options;
        return `${namespace ? namespace + NAMESPACE_DELIM : ''}${name}`;
    }
    /**
     * Parse usage text into usable table structure
     * @param usage the plain usage text emitted by yeoman
     */
    _tabular(usage) {
        return usage.trim().split(new RegExp(os_1.EOL + '+'))
            .filter(line => !!line)
            .map(line => {
            return line.trim()
                .replace(/(\-\w+),\s+([-\w]+)/, '$1, $2') // flags
                .split(/\s{2,}/) // tabs
                .map(col => col.replace(/^\#\s/, '')); // remove leading #
        });
    }
    /**
     * gets yeoman env
     */
    _env() {
        const { _adapter } = this.constructor;
        const env = this._yo || YeomanEnv.createEnv([], {}, _adapter);
        this._yo = env;
        return env;
    }
    /**
     * Ensures the environment has been loaded, otherwise throws error
     * @param method method name being used
     * @throws when no environment is loaded
     */
    _ensureEnv(method) {
        if (!this._yo) {
            throw new Error(`Generators must be loaded before calling ${this.constructor.name}::${method}`);
        }
        return this;
    }
};
GeneratorEnv = GeneratorEnv_1 = __decorate([
    cli_1.Provide({
        factory: (from) => GeneratorEnv_1.relativeTo(from || cli_1.Workspace.commandsRoot()),
    }),
    __metadata("design:paramtypes", [Object])
], GeneratorEnv);
exports.GeneratorEnv = GeneratorEnv;
